(function(angular, ionic, undefined) {

    angular
        .module('headsup')
        .controller('ApplicationMainController', ApplicationMainController);

    ApplicationMainController.$inject = ['$userAccount', '$state', '$deviceManager', '$rootScope', '$ionicHistory'];

    function ApplicationMainController($userAccount, $state, $deviceManager, $rootScope, $ionicHistory) {

        var vm = this;

        vm.login = login;
        vm.logout = logout;

        $rootScope.$on('headsup:user:logout', resolveGlobalStateVariables);
        $rootScope.$on('headsup:user:login', resolveGlobalStateVariables);
        $rootScope.$on('$stateChangeSuccess', resolveGlobalStateVariables);

        resolveGlobalStateVariables();

        function login() {
            $state.go('app.login');
        };

        function logout() {

            $userAccount.logout()
                .then(function() {
                    $deviceManager.alert('You have been successfully logged out.');


                    $ionicHistory.nextViewOptions({
                        disableBack: true
                    });

                    $state.go('app.alerts', {}, { reload: true });

                }, function() {
                    throw 'Could not logout user right now.';
                });
        };

        function resolveGlobalStateVariables() {
            vm.authenticated = $userAccount.isAuthenticated();
            vm.username = $userAccount.getSavedUsername();
            vm.allowCreateAlert = vm.authenticated && $state.current.data && $state.current.data.actions && $state.current.data.actions.allowCreateAlert;
            vm.fatalError = $state.current.data && $state.current.data.fatalError;
            vm.app = applicationConfiguration.app;
        };
    };

})(angular, ionic);

