(function(angular, ionic, undefined) {

    angular
        .module('headsup')
        .directive('multiSelect', multiSelect);

    multiSelect.$inject = ['$ionicModal', '$compile', '$q'];

    function multiSelect($ionicModal, $compile, $q) {

        var directive = {
            templateUrl: "app/directives/multi-select/multi-select.directive.html",
            restrict: 'E',
            scope: {
                options: '=?',
                source: '&?',
                ngModel: '=',
                placeholder: '=?'
            },
            link: link,
            transclude: true
        };

        return directive;

        function link(scope, element, attrs, ctrl, transclude) {

            var
                contentPlaceholder = element.find('multi-select-modal-content'),
                selectionPlaceholder = element.find('multi-select-selection'),
                modal = buildModal(contentPlaceholder, scope);

            if (!scope.placeholder) {
                scope.placeholder = 'None selected.';
            };

            scope.closeModal = closeModal;
            scope.openModal = openModal;

            element.on('click', openModal);

            activate();

            function closeModal() {
                scope.modal.hide();
            };

            function openModal() {
                modal.then(function() {
                    scope.modal.show();
                });
            };

            function activate() {

                var
                    idProperty = attrs.optionIdProperty ? attrs.optionIdProperty : 'id',
                    labelProperty = attrs.optionLabelProperty ? attrs.optionLabelProperty : 'label';

                if (scope.options) {

                    modal.then(function() {

                        scope.$watchCollection('options', function() {
                            renderSelectionList(scope, contentPlaceholder, selectionPlaceholder, scope.options, idProperty, labelProperty, transclude);
                        });
                    });
                } else if (scope.source) {

                    modal.then(function() {

                        scope.refreshOptionsSource = refresh;

                        function refresh() {

                            scope
                                .source()
                                .then(function(options) {
                                    renderSelectionList(scope, contentPlaceholder, selectionPlaceholder, options, idProperty, labelProperty, transclude);
                                    scope.$broadcast('scroll.refreshComplete');
                                });
                        };

                        refresh();
                    });
                } else {
                    throw 'Options nor options source is provided.'
                };
            };

        };

        function buildModal(content, scope) {

            var
                deferred = $q.defer();

            $ionicModal.fromTemplateUrl('app/directives/multi-select/modal.html', {
                scope: scope,
                animation: 'slide-in-up'
            }).then(function(modal) {
                scope.modal = modal;
                angular.element(modal.modalEl).find('ionic-modal-transcluded-content-placeholder').append(content);
                deferred.resolve();
            });

            return deferred.promise;
        };

        function renderSelectionList(parentScope, contentPlaceholder, selectionPlaceholder, options, optionIdProperty, optionLabelProperty, transclude) {

            contentPlaceholder.empty();
            selectionPlaceholder.text(parentScope.placeholder);

            var
                selected = []

            for (var i = 0; i < options.length; i++) {

                var
                    option = options[i],
                    childScope = parentScope.$new(),
                    item = angular.element('<ion-item ng-click="select(option, $event)"><option-content></option-content><i class="checkbox-icon"></i></ion-item>');

                childScope.option = option;

                if (parentScope.ngModel.indexOf(option[optionIdProperty]) > -1) {
                    item.addClass('selected');

                    if (angular.isFunction(option[optionLabelProperty])) {
                        selected.push(option[optionLabelProperty]());
                    } else {
                        selected.push(option[optionLabelProperty]);
                    };
                };

                childScope.select = function(option, $event) {

                    var
                        index = parentScope.ngModel.indexOf(option[optionIdProperty]);

                    if (index > -1) {
                        parentScope.ngModel.splice(index, 1);
                        angular.element($event.currentTarget).removeClass('selected');
                    } else {
                        parentScope.ngModel.push(option[optionIdProperty]);
                        angular.element($event.currentTarget).addClass('selected');
                    };

                    renderSelectionLabel(parentScope, selectionPlaceholder, options, optionIdProperty, optionLabelProperty);
                };


                transclude(parentScope, function(clone) {

                    if (!clone.text().trim()) {

                        if (angular.isFunction(option[optionLabelProperty])) {
                            item.find('option-content').text(option[optionLabelProperty]());
                        } else {
                            item.find('option-content').text(option[optionLabelProperty]);
                        };
                    } else {
                        item.find('option-content').append(clone);
                    };
                });

                contentPlaceholder.append($compile(item)(childScope));
            };

            if (selected.length > 0) {
                selectionPlaceholder.text(selected.length + ' selected (' + selected.join(', ') + ')');
            };
        };

        function renderSelectionLabel(parentScope, selectionPlaceholder, options, optionIdProperty, optionLabelProperty) {

            var
                selected = []

            if (parentScope.ngModel.length > 0) {

                for (var i = 0; i < options.length; i++) {

                    var
                        option = options[i];


                    if (parentScope.ngModel.indexOf(option[optionIdProperty]) > -1) {

                        if (angular.isFunction(option[optionLabelProperty])) {
                            selected.push(option[optionLabelProperty]());
                        } else {
                            selected.push(option[optionLabelProperty]);
                        };
                    };
                };

                if (selected.length > 0) {
                    selectionPlaceholder.text(selected.length + ' selected (' + selected.join(', ') + ')');
                };
            } else {
                selectionPlaceholder.text(parentScope.placeholder);
            }
        };
    };

})(angular, ionic);



