(function(angular, ionic, window, undefined) {

    angular
        .module('headsup')
        .service('$localStorage', LocalStorage);

    function LocalStorage() {

        var
            self = this,
            cache = {};

        this.get = function(key, defaultValue) {

            if (cache.hasOwnProperty(key)) {
                return cache[key];
            };

            if (window.localStorage.hasOwnProperty(key)) {
                var
                    value = JSON.parse(window.localStorage[key]).value;

                if (undefined !== value) {
                    return (cache[key] = value);
                };
            };

            return defaultValue;
        };

        this.has = function(key) {
            return cache.hasOwnProperty(key) || window.localStorage.hasOwnProperty(key);
        };

        this.save = function(key, value) {

            window.localStorage[key] = JSON.stringify({
                value: value
            });

            cache[key] = value;

            return self;
        };

        this.delete = function(key) {
            window.localStorage.removeItem(key);
            delete cache[key];

            return self;
        };

    };

})(angular, ionic, window);

