(function(angular, ionic, undefined) {

    angular
        .module('headsup')
        .service('$pushNotifications', PushNotifications);

    PushNotifications.$inject = ['$q', '$http', '$rootScope', '$deviceManager', '$utilities', '$ionicPush'];

    function PushNotifications($q, $http, $rootScope, $deviceManager, $utilities, $ionicPush) {

        var
            self = this,
            initialized = false,
            pushNotificationsToken = undefined
            ;

        this.init = init;
        this.enabled = enabled;
        this.isRegistered = isRegistered;
        this.ready = ready;

        function init() {

            if (!initialized) {

                $ionicPush.init({
                    debug: false,
                    onRegister: function(data) {

                        var
                            deferred = (pushNotificationsToken && undefined !== pushNotificationsToken.promise) ? pushNotificationsToken : $q.defer();

                        $http({
                            method: 'POST',
                            url: $utilities.getApiUrl('/device/token/v1'),
                            headers: {
                                'Content-Type': 'application/json'
                            },
                            data: {
                                "deviceType": $deviceManager.getPlatform(),
                                "deviceId": $deviceManager.getUuid(),
                                "deviceToken": data.token,
                                "deviceApp": $deviceManager.getApp()
                            }
                        }).then(function(result) {
                            pushNotificationsToken = (result.data.error == false) ? data.token : null;
                            deferred.resolve();
                        }).catch(function() {
                            pushNotificationsToken = null;
                            deferred.reject();
                        });
                    },
                    pluginConfig: {
                        ios: {
                            badge: true,
                            sound: true,
                            clearBadge: true
                        },
                        android: {
                            sound: true,
                            vibrate: true,
                            clearNotifications: true
                        }
                    },
                    onNotification: function(notification) {
                        $rootScope.$broadcast('pushNotification:received', notification._raw);
                    }
                });

                $ionicPush.register();

                initialized = true;
            };

            return self;
        };

        function enabled() {

            var
                deferred = $q.defer();

            if (null === push) {
                deferred.reject();
            } else {

                PushNotification.hasPermission(function(data) {

                    if (data.isEnabled) {
                        deferred.resolve();
                    } else {
                        deferred.reject();
                    };
                });
            };

            return deferred.promise;
        };

        function isRegistered() {
            return undefined !== pushNotificationsToken;
        };

        function ready() {

            var
                deferred = $q.defer();

            if (null === push) {

                deferred.reject();

            } else {

                if (pushNotificationsToken && undefined !== pushNotificationsToken.promise) {
                    return pushNotificationsToken.promise;
                }


                if (!self.isRegistered() || null === pushNotificationsToken) {
                    deferred.reject();
                } else {
                    pushNotificationsToken = deferred;
                };
            }

            return deferred.promise;
        };
    };

})(angular, ionic);

