(function(angular, ionic, undefined) {

    angular
        .module('headsup')
        .service('$utilities', Utilities);

    Utilities.$inject = ['$configuration'];

    function Utilities($configuration) {

        var
            self = this;

        this.getApiUrl = getApiUrl;
        this.processSettingsTree = processSettingsTree;
        this.getCenterOfGeomarkersCollection = getCenterOfGeomarkersCollection;
        this.getPolygonCenter = getPolygonCenter;
        this.objectPropertiesStringToDate = objectPropertiesStringToDate;
        this.objectPropertiesDateToString = objectPropertiesDateToString;

        function getApiUrl(path) {
            return $configuration.get('serverAddress') + $configuration.get('apiSubPath') + path;
        };

        function processSettingsTree(treeOrNode, type, subscriptions) {

            if (treeOrNode.constructor === Array) {

                angular.forEach(treeOrNode, function(node){
                    self.processSettingsTree(node, type);
                });

            } else {

                treeOrNode.type = type;

                if (treeOrNode.children && treeOrNode.children.length > 0) {

                    angular.forEach(treeOrNode.children, function(child) {

                        child.parent = treeOrNode;

                        if (child.icon === null && treeOrNode.icon) {
                            child.icon = treeOrNode.icon;
                        };

                        if (child.marker === null && treeOrNode.marker) {
                            child.marker = treeOrNode.marker;
                        };

                        self.processSettingsTree(child, type);
                    });
                };
            };

            return treeOrNode;
        };

        function getCenterOfGeomarkersCollection(geomarkers) {

            var
                points = [];

            angular.forEach(geomarkers, function(geomarker) {

                if ('rectangle' === geomarker.type) {
                    points.push(geomarker.northEast);
                    points.push(geomarker.southWest);
                } else if ('circle' === geomarker.type) {
                    points.push(geomarker.center);
                } else if ('polygon' === geomarker.type) {
                    points.concat(geomarker.points);
                } else {
                    points.push(geomarker);
                }
            });

            return getPolygonCenter(points);
        };

        function getPolygonCenter(coordinates) {

            var
                minLat = 1000,
                maxLat = -1000,
                minLng = 1000,
                maxLng = -1000;

            angular.forEach(coordinates, function(coordinate) {
                minLat = Math.min(minLat, coordinate.lat);
                minLng = Math.min(minLng, coordinate.lng);

                maxLat = Math.max(maxLat, coordinate.lat);
                maxLng = Math.max(maxLng, coordinate.lng);
            });

            return {
                lat: minLat + ((maxLat - minLat) / 2),
                lng: minLng + ((maxLng - minLng) / 2)
            };
        };

        function objectPropertiesStringToDate(object, fields) {

            if (!angular.isArray(fields)) {
                fields = [fields];
            };

            angular.forEach(fields, function(field) {
                object[field] = (object[field]) ? moment(object[field] + ' +0000', 'YYYY-MM-D\THH:mm Z').toDate() : null;
            });
        };

        function objectPropertiesDateToString(object, fields) {

            if (!angular.isArray(fields)) {
                fields = [fields];
            };

            angular.forEach(fields, function(field) {
                object[field] = object[field] ?  moment(object[field]).utc().format('YYYY-MM-DD HH:mm') : null;
            });
        };

    };

})(angular, ionic);

