(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .config(routeConfigurator);

    routeConfigurator.$inject = ['$stateProvider'];

    function routeConfigurator($stateProvider) {

        $stateProvider
            .state('app.alerts', {
                url: '/alerts',
                cache: true,
                data: {
                    actions: {
                        allowCreateAlert: true
                    }
                },
                views: {
                    'menuContent': {
                        templateUrl: 'modules/alerts/view/list.controller.html',
                        controller: 'ListAlertsController as vm'
                    }
                },
                resolve: {
                    initialize: ['$deviceManager', function ($deviceManager) {
                        return $deviceManager.initiate();
                    }]
                }
            })
            .state('app.alert', {
                url: '/alerts/:alertId',
                cache: true,
                data: {
                    actions: {
                        allowCreateAlert: true
                    }
                },
                views: {
                    'menuContent': {
                        controller: 'ShowAlertController as vm',
                        templateUrl: 'modules/alerts/view/show.controller.html'
                    }
                },
                resolve: {
                    initialize: ['$deviceManager', '$geomarking', '$q', function ($deviceManager, $geomarking, $q) {

                        var
                            deferred = $q.defer();

                        $deviceManager
                            .initiate()
                            .then(function() {
                                return $geomarking.loadGoogleMapLibrary();
                            })
                            .then(function() {
                                deferred.resolve();

                            });

                        return deferred.promise;
                    }]
                }
            })
            .state('app.myAlerts', {
                url: '/alerts/my-alerts',
                cache: true,
                data: {
                    actions: {
                        allowCreateAlert: true
                    }
                },
                views: {
                    'menuContent': {
                        templateUrl: 'modules/alerts/view/my-alerts.controller.html',
                        controller: 'MyAlertsController as vm'
                    }
                },
                resolve: {
                    initialize: ['$deviceManager', function ($deviceManager) {
                        return $deviceManager.initiate();
                    }]
                }
            })
            .state('app.createAlert', {
                url: '/alerts/create',
                views: {
                    'menuContent': {
                        controller: 'EditAlertController as vm',
                        templateUrl: 'modules/alerts/view/create-alert.controller.html'
                    }
                },
                resolve: {
                    initialize: ['$deviceManager', '$geomarking', '$q', '$alerts', function ($deviceManager, $geomarking, $q, $alerts) {

                        var
                            deferred = $q.defer();

                        $deviceManager
                            .initiate()
                            .then(function() {
                                return $alerts.loadMetadata();
                            })
                            .then(function() {
                                return $geomarking.loadGoogleMapLibrary();
                            })
                            .then(function() {
                                deferred.resolve();
                            });

                        return deferred.promise;
                    }]
                }
            })
            .state('app.editAlert', {
                url: '/alerts/edit/:alertId',
                views: {
                    'menuContent': {
                        controller: 'EditAlertController as vm',
                        templateUrl: 'modules/alerts/view/edit-alert.controller.html'
                    }
                },
                resolve: {
                    initialize: ['$deviceManager', '$geomarking', '$q', '$alerts', function ($deviceManager, $geomarking, $q, $alerts) {

                        var
                            deferred = $q.defer();

                        $deviceManager
                            .initiate()
                            .then(function() {
                                return $alerts.loadMetadata();
                            })
                            .then(function() {
                                return $geomarking.loadGoogleMapLibrary();
                            })
                            .then(function() {
                                deferred.resolve();
                            });

                        return deferred.promise;
                    }]
                }
            })
        ;

    };

})(angular, ionic);


