(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .controller('EditAlertController', EditAlertController);

    EditAlertController.$inject = ['$alerts', '$locations', '$q', '$ionicScrollDelegate', '$ionicLoading', '$state', '$ionicHistory', '$stateParams', '$scope', '$deviceManager', 'Upload', '$ionicListDelegate', '$controller'];

    function EditAlertController($alerts, $locations, $q, $ionicScrollDelegate, $ionicLoading, $state, $ionicHistory, $stateParams, $scope, $deviceManager, Upload, $ionicListDelegate, $controller) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var vm = this;

        vm.error = {
            form: false,
            save: false
        };

        if ($locations.ids().length === 1) {
            vm.disableLocationsSelection = true;
        };

        vm.getAlerts = getAlerts;
        vm.submit = submit;
        vm.clearValue = clearValue;

        $scope.$on('$ionicView.enter', onIonicViewEnter);

        function onIonicViewEnter() {

            vm.alert = null;

            $alerts
                .loadMetadata()
                .then(function(metadata) {

                    vm.metadata = metadata;

                    var
                        deferred = $q.defer();

                    if ($stateParams.alertId) {

                        $alerts
                            .getMy($stateParams.alertId)
                            .then(function (alert) {
                                vm.alert = alert;
                                deferred.resolve();
                            });

                    } else {
                        vm.alert = $alerts.new();
                        deferred.resolve();
                    };

                    return deferred.promise;
                })
                .then(function(metadata) {
                    if ($locations.ids().length === 1) {
                        vm.alert.audience = $locations.ids()[0];
                        vm.disableLocationsSelection = true;
                    };
                });
        };

        function getAlerts() {
            var
                deferred = $q.defer();

            $alerts
                .list()
                .then(function(response){
                    deferred.resolve(response.data);
                });

            return deferred.promise;
        };

        function submit(form) {

            vm.error.form = false;
            vm.error.save = false;

            if (form.$valid) {

                $ionicLoading.show({
                    content: 'Loading',
                    animation: 'fade-in',
                    showBackdrop: true,
                    showDelay: 0
                });

                $q
                    .when(preProcessAlert(vm.alert))
                    .then(function(alert) {
                        return $alerts.save(alert);
                    })
                    .then(function(id) {

                        $deviceManager.setAlertRead(id); // What we have created/edited, it is seen.

                        $ionicHistory.nextViewOptions({
                            disableBack: true
                        });

                        $state.go('app.myAlerts', {}, { reload: true });
                        $scope.$emit('user:flash:success', 'Alert is successfully saved.');

                        vm.alert = null;

                        return $q.resolve();
                    })
                    .catch(function() {
                        vm.error.save = true;
                        $ionicScrollDelegate.scrollTop(true);
                    })
                    .finally(function() {
                        $ionicLoading.hide();
                    });

                return;
            };

            vm.error.form = true;
            $ionicScrollDelegate.scrollTop(true);
        };

        function clearValue(object, key) {

            if (object && undefined !== object[key]) {
                object[key] = null;
            } else {
                throw 'Unknown property: ' + key;
            };

            $ionicListDelegate.closeOptionButtons();

            return false;
        };

        function preProcessAlert(alert) {

            var
                deffered = $q.defer();

            if (alert.scope != vm.metadata.fields.scope.restrictedValue) {
                alert.restriction = null;
            };

            if (alert.scope != vm.metadata.fields.scope.privateValue) {
                alert.scopeAddress = [];
            };

            var
                validAdresses = [],
                validInstructions = [],
                i = 0,
                promiseChain = $q.resolve();

            for (i = 0; i < alert.scopeAddress.length; i++) {

                if (alert.scopeAddress[i].trim() !== '') {
                    validAdresses.push(alert.scopeAddress[i].trim());
                };
            };

            alert.scopeAddress = validAdresses;


            for (i = 0; i < alert.instruction.length; i++) {

                if (alert.instruction[i].trim() !== '') {
                    validInstructions.push(alert.instruction[i].trim());
                };
            };

            alert.instruction = validInstructions;

            angular.forEach(alert.images, function(image) {

                if (image.id === null && image.data === null) {

                    promiseChain = promiseChain.then(function() {
                        var
                            innerDeferred = $q.defer();

                        Upload.base64DataUrl(image.src).then(function(data) {
                            image.data = data;
                            innerDeferred.resolve();
                        });

                        return innerDeferred.promise;
                    });
                };
            });

            angular.forEach(alert.files, function(file) {

                if (file.id === null && file.data === null) {

                    promiseChain = promiseChain.then(function() {
                        var
                            innerDeferred = $q.defer();

                        Upload.base64DataUrl(file.src).then(function(data) {
                            file.data = data;
                            innerDeferred.resolve();
                        });

                        return innerDeferred.promise;
                    });
                };
            });


            promiseChain.then(function() {
                deffered.resolve(alert);
            });

            return deffered.promise;
        };

    };

})(angular, ionic);
