(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .controller('ListAlertsController', ListAlertsController);

    ListAlertsController.$inject = ['$alerts', '$deviceManager', '$ionicHistory', '$scope', '$controller'];

    function ListAlertsController($alerts, $deviceManager, $ionicHistory, $scope, $controller) {

        var
            vm = this,
            hasMore = false,
            limit = 15;

        vm.load = load;
        vm.isHidden = isHidden;
        vm.moreDataCanBeLoaded = moreDataCanBeLoaded;
        vm.alerts = [];

        $scope.$on('$ionicView.enter', onIonicViewEnter);

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { vm: vm, alerts: 'alerts', withPopup: false }}));

        function load(append) {

            hasMore = false;

            vm.nbSubscribedCategories = $deviceManager.getSubscribedCategories().length;
            vm.nbSubscribedLocations = $deviceManager.getSubscribedLocations().length;

            if (vm.nbSubscribedCategories > 0 && vm.nbSubscribedLocations > 0) {

                $alerts.list({
                    offset: (append) ? vm.alerts.length : 0,
                    limit: limit
                }).then(function(alerts) {

                    vm.loadingSpinner = false;

                    if (!append) {
                        vm.alerts = [];
                        $scope.$emit('alerts:latest-loaded');
                    };

                    hasMore = alerts.hasMore;

                    angular.forEach(alerts.data, function(alert) {

                        if (!vm.isHidden(alert)) {
                            vm.alerts.push(alert);
                        };
                    });
                });

            } else {
                vm.loadingSpinner = false;
                vm.alerts = [];
            };

            $scope.$broadcast((append) ? 'scroll.infiniteScrollComplete' : 'scroll.refreshComplete');
        };

        function moreDataCanBeLoaded() {
            return hasMore;
        };

        function isHidden(alert) {
            return $deviceManager.isAlertHidden(alert.id);
        };

        function onIonicViewEnter() {

            if (!$ionicHistory.forwardView()
                ||
                (
                    $ionicHistory.forwardView().stateName !== 'app.alert'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.login'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.createAlert'
                )
            ) {
                vm.load();
                vm.loadingSpinner = true;
            };
        };
    };

})(angular, ionic);
