(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .controller('MyAlertsController', MyAlertsController);

    MyAlertsController.$inject = ['$alerts', '$ionicHistory', '$scope', '$controller'];

    function MyAlertsController($alerts, $ionicHistory, $scope, $controller) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var
            vm = this,
            hasMore = false,
            limit = 15;

        vm.reload = reload;
        vm.loadMore = loadMore;
        vm.moreDataCanBeLoaded = moreDataCanBeLoaded;

        $scope.$on('$ionicView.enter', onIonicViewEnter);

        function reload() {
            hasMore = false;

            $alerts.listMy({
                offset: 0,
                limit: limit
            }).then(function(alerts) {

                hasMore = alerts.hasMore;

                vm.loadingSpinner = false;
                vm.alerts = alerts.data;
            });

            $scope.$broadcast('scroll.refreshComplete');
        };

        function loadMore() {

            $alerts.listMy({
                offset: vm.alerts.length,
                limit: limit
            }).then(function(alerts) {

                hasMore = alerts.hasMore;

                angular.forEach(alerts.data, function(alert) {
                    vm.alerts.push(alert);
                });

                $scope.$broadcast('scroll.infiniteScrollComplete');
            });
        };

        function moreDataCanBeLoaded() {
            return hasMore;
        };

        function onIonicViewEnter() {

            if (!$ionicHistory.forwardView()
                ||
                (
                    $ionicHistory.forwardView().stateName !== 'app.alert'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.login'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.editAlert'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.createAlert'
                )
            ) {
                vm.reload();
                vm.loadingSpinner = true;
            };
        };
    };

})(angular, ionic);
