(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .controller('PushedAlertAbstractController', PushedAlertAbstractController);

    PushedAlertAbstractController.$inject = ['options', '$scope', '$ionicPopover', '$state', '$pushedAlertsManager', '$alerts', '$q', '$deviceManager'];

    function PushedAlertAbstractController(options, $scope, $ionicPopover, $state, $pushedAlertsManager, $alerts, $q, $deviceManager) {

        options = angular.extend({
            withPopup: true,
            vm: null,
            alerts: null
        }, options);

        var
            popover = null,
            vm = options.vm ? options.vm : this;

        vm.upcomingAlertsCount = $pushedAlertsManager.count();

        vm.loadLatestAlerts = loadLatestAlerts;

        $scope.$on('alerts:notification-stack:empty', onAlertsNotificationStackEmpty);
        $scope.$on('alerts:notification-stack:push', onAlertsNotificationStackPush);

        function loadLatestAlerts() {

            $pushedAlertsManager.clear();

            if (!options.alerts || !vm[options.alerts]) {
                return;
            };

            var
                latestDate = new Date(0);

            if (vm[options.alerts].length > 0) {
                latestDate = new Date(vm[options.alerts][0].created.getTime());
            };

            $alerts
                .latest(latestDate)
                .then(function(response) {

                    if (response.hasMore) {
                        vm[options.alerts].splice(0, alerts.length);
                    };

                    response.data.reverse();

                    angular.forEach(response.data, function(alert) {
                        var
                            exists = false;

                        for (var i = 0; i < vm[options.alerts].length; i++) {

                            if (vm[options.alerts][i].id == alert.id) {
                                vm[options.alerts][i] = alert;
                                $deviceManager.setAlertUnread(alert.id);
                                exists = true;
                                break;
                            };
                        };

                        if (!exists) {
                            vm[options.alerts].unshift(alert);
                        };
                    });

                    vm[options.alerts].sort(function(a1, a2) {
                        return (a1.created > a2.created) ? -1 : 1;
                    });
                });
        };

        function onAlertsNotificationStackEmpty($event) {
            vm.upcomingAlertsCount = $pushedAlertsManager.count();
            $scope.$evalAsync();
        };

        function onAlertsNotificationStackPush($event, alert) {
            vm.upcomingAlertsCount = $pushedAlertsManager.count();
            $scope.$evalAsync();

            if (options.withPopup) {
                showNewAlertPopup(alert);
            };
        };

        function showNewAlertPopup(alert) {

            $q
                .when()
                .then(function() {

                    var
                        deferred = $q.defer();

                    if (popover) {

                        popover
                            .hide()
                            .then(function() {
                                popover.remove();
                                popover = null;
                                return deferred.resolve();
                            });

                    } else {
                        deferred.resolve();
                    };

                    return deferred.promise;
                })
                .then(function() {
                    var
                        popoverScope = $scope.$new();

                    popoverScope.alert = alert;
                    popoverScope.show = gotoAlertShow;
                    popoverScope.close = closePopup;

                    $ionicPopover.fromTemplateUrl('modules/alerts/view/partials/pushed-alert.popover.html', {
                        scope: popoverScope,
                        animation: 'slide-in-up'
                    }).then(function(result){
                        popover = result;
                        popover.show();
                    });
                });
        };

        function gotoAlertShow(alert) {

            if (popover) {
                popover.remove();
                popover = null;
            };

            $state.go('app.alert', {
                alertId: alert.id
            });
        };

        function closePopup() {

            if (popover) {
                popover
                    .hide()
                    .then(function() {
                        popover.remove();
                        popover = null;
                    });

            };
        };
    };

})(angular, ionic);

