(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .controller('ShowAlertController', ShowAlertController);

    ShowAlertController.$inject = ['$stateParams', '$alerts', '$cordovaInAppBrowser', '$ionicHistory', '$scope', '$deviceManager', '$controller'];

    function ShowAlertController($stateParams, $alerts, $cordovaInAppBrowser, $ionicHistory, $scope, $deviceManager, $controller) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var vm = this;

        vm.reload = reload;
        vm.launch = launch;

        $scope.$on('$ionicView.enter', onIonicViewEnter);
        $scope.$on('$ionicView.afterLeave', onIonicViewAfterLeave);

        function reload() {

            $alerts
                .get($stateParams.alertId)
                .then(function(alert) {
                    $deviceManager.setAlertRead(alert.id);
                    vm.loadingSpinner = false;
                    vm.alert = alert;
                })
                .catch(function() {
                    $ionicHistory.goBack();
                    $scope.$emit('user:flash:error', 'Requested alert is not accessible anymore.');
                })
                .finally(function() {
                    $scope.$broadcast('scroll.refreshComplete');
                });
        };

        function launch(url) {
            $cordovaInAppBrowser.open(url, '_blank', {});
        };

        function onIonicViewEnter() {

            if (
                !$ionicHistory.forwardView()
                ||
                (
                    $ionicHistory.forwardView().stateName !== 'app.asset'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.alert'
                )
                ||
                !vm.alert
            ) {
                vm.reload();
                vm.loadingSpinner = true;
            };
        };

        function onIonicViewAfterLeave() {

            if ($ionicHistory.currentView() && $ionicHistory.currentView().stateName == 'app.alerts') {
                vm.alert = null;
            };
        };
    };

})(angular, ionic);

