(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .directive('alertCategoryChoice', alertCategoryChoice);

    alertCategoryChoice.$inject = ['$categories', '$ionicModal', '$ionicScrollDelegate', '$alerts'];

    function alertCategoryChoice($categories, $ionicModal, $ionicScrollDelegate, $alerts) {

        var directive = {
            templateUrl: "modules/alerts/directives/alert-category-choice/alert-category-choice.html",
            restrict: 'E',
            scope: {
                ngModel: '=',
                placeholder: '=?'
            },
            compile: compile
        };

        return directive;

        function compile(element, attrs, transclude) {

            if (attrs.required !== undefined) {
                element.find('input').attr('required', 'required');
            };

            return link;
        };

        function link(scope, element, attrs) {

            scope.name = (attrs.name !== undefined) ? attrs.name : 'category';
            scope.categories = null;

            $alerts
                .loadMetadata()
                .then(function (result) {
                    scope.categories = result.fields.category.choices;
                });

            scope.closeModal = closeModal;
            scope.openModal = openModal;
            scope.icon = icon;
            scope.label = label;
            scope.select = select;
            scope.toggleOpenState = toggleOpenState;

            element.on('click', openModal);

            $ionicModal.fromTemplateUrl('alert-category-choice-modal.html', {
                scope: scope,
                animation: 'slide-in-up'
            }).then(function(modal) {
                scope.modal = modal;
            });

            function select(value) {
                scope.ngModel = (scope.ngModel == value) ? null : value;

                if (scope.ngModel) {
                    closeModal();
                };
            };

            function openModal() {
                scope.modal.show();
            };

            function closeModal() {
                scope.modal.hide();
            };

            function icon(id) {
                return $categories.icon(id);
            };

            function label(id) {
                return $categories.label(id);
            };

            function toggleOpenState($event) {

                var
                    container = angular.element($event.currentTarget).toggleClass('closed').next().toggleClass('closed');

                if (container.hasClass('closed')) {

                    container.slideUp(function() {
                        $ionicScrollDelegate.resize();
                    });
                } else {

                    container.slideDown(function() {
                        $ionicScrollDelegate.resize();
                    });
                };
            };
        };
    };

})(angular, ionic);


