(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .directive('alertHeader', alertHeader);

    alertHeader.$inject = ['$state', '$cordovaSocialSharing', '$deviceManager', '$window'];

    function alertHeader($state, $cordovaSocialSharing, $deviceManager, $window) {

        var directive = {
            templateUrl: "modules/alerts/directives/alert-header/alert-header.directive.html",
            restrict: 'E',
            scope: {
                alert: '=',
                action: '@'
            },
            link: link
        };

        return directive;

        function link(scope, element, attrs) {

            var
                now = new Date(),
                schedule = new Date(scope.alert.created);

            scope.scheduled = schedule >= now;
            scope.allowAddToCalendar = false;

            scope.onAction = onAction;
            scope.addToCalendar = addToCalendar;
            scope.canBeAddedToCalendar = canBeAddedToCalendar;

            function onAction(alert) {

                if (scope.action && 'share' === scope.action) {

                    if ($deviceManager.isBrowser()) {
                        $deviceManager.alert('Sharing:\n\nTitle: Heads Up\nTitle: ' + alert.title + '\nUrl: ' + alert.url);
                    } else {
                        $cordovaSocialSharing.share(alert.title, 'Heads Up', null, alert.url);
                    };
                } else {

                    $state.go('app.alert', {
                        alertId: alert.id
                    });
                };
            };

            function canBeAddedToCalendar(alert) {

                if (!alert.eventOnset) {
                    return false;
                };

                if ($deviceManager.isBrowser()) {
                    return false;
                };

                var
                    now = new Date(),
                    eventOnset = new Date(alert.eventOnset);

                if (eventOnset <= now) {
                   return false;
                };

                return true;
            };

            function addToCalendar(alert) {

                if (canBeAddedToCalendar(alert)) {

                    $window.plugins.calendar.createEventInteractivelyWithOptions(
                        alert.title,
                        null,
                        alert.details,
                        new Date(alert.eventOnset),
                        (alert.eventExpires) ? new Date(alert.eventExpires) : new Date(alert.eventOnset),
                        {
                            url: alert.url
                        }
                    );
                };
            };

        };
    };

})(angular, ionic);

