(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .directive('alertListItem', alertListItem);

    alertListItem.$inject = ['$state', '$cordovaSocialSharing', '$deviceManager', '$ionicListDelegate', '$alerts', '$ionicPopup'];

    function alertListItem($state, $cordovaSocialSharing, $deviceManager, $ionicListDelegate, $alerts, $ionicPopup) {

        var directive = {
            templateUrl: "modules/alerts/directives/alert-list-item/alert-list-item.directive.html",
            restrict: 'E',
            scope: {
                alert: '=',
                withPublish: '=?',
                withUnpublish: '=?',
                withEdit: '=?',
                withShare: '=?',
                withRead: '=?',
                withUnread: '=?',
                withHide: '=?'
            },
            link: link
        };

        return directive;

        function link(scope, element, attrs) {

            var
                now = new Date(),
                schedule = new Date(scope.alert.created);

            scope.scheduled = (schedule >= now);

            scope.show = show;
            scope.share = share;
            scope.publish = publish;
            scope.unpublish = unpublish;
            scope.edit = edit;
            scope.isRead = isRead;
            scope.read = read;
            scope.unread = unread;
            scope.hide = hide;


            function show(alert) {
                $ionicListDelegate.closeOptionButtons();

                $state.go('app.alert', {
                    alertId: alert.id
                });
            };

            function share(alert) {

                if ($deviceManager.isBrowser()) {
                    $deviceManager.alert('Sharing:\n\nTitle: Heads Up\nTitle: ' + alert.title + '\nUrl: ' + alert.url);
                } else {
                    $cordovaSocialSharing.share(alert.title, 'Heads Up', null, alert.url);
                };

                $ionicListDelegate.closeOptionButtons();
            };

            function publish(alert) {

                $alerts
                    .publish(alert.id)
                    .then(function() {
                        alert.published = true;
                    });

                $ionicListDelegate.closeOptionButtons();
            };

            function unpublish(alert) {

                $alerts
                    .unpublish(alert.id)
                    .then(function() {
                        alert.published = false;
                    });

                $ionicListDelegate.closeOptionButtons();
            };

            function edit(alert) {
                $ionicListDelegate.closeOptionButtons();

                $state.go('app.editAlert', {
                    alertId: alert.id
                }, {
                    reload: true
                });
            };

            function isRead(alert) {
                return $deviceManager.isAlertRead(alert.id);
            };

            function read(alert) {
                $deviceManager.setAlertRead(alert.id);
                $ionicListDelegate.closeOptionButtons();
            };

            function unread(alert) {
                $deviceManager.setAlertUnread(alert.id);
                $ionicListDelegate.closeOptionButtons();
            };

            function hide(alert) {

                $ionicPopup.confirm({
                    title: 'Remove alert',
                    template: '<div class="text-center">Are you sure you want to remove <br/><em>"' + alert.title + '"</em><br/> from the list? <br/><br/>This can not be undone.</div>'
                }).then(function(result) {

                    if (result) {
                        element
                            .css({
                                'position': 'relative'
                            })
                            .animate({
                                'left': '-100%'
                            }, 200, function() {
                                element.remove();
                                $deviceManager.hideAlert(alert.id);
                            });
                    };

                    $ionicListDelegate.closeOptionButtons();
                });
            };
        };
    };

})(angular, ionic);

