(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .directive('alertLocationChoice', alertLocationChoice);

    alertLocationChoice.$inject = ['$ionicModal', '$ionicScrollDelegate', '$alerts'];

    function alertLocationChoice($ionicModal, $ionicScrollDelegate, $alerts) {

        var directive = {
            templateUrl: "modules/alerts/directives/alert-location-choice/alert-location-choice.directive.html",
            restrict: 'E',
            scope: {
                ngModel: '=',
                placeholder: '=?'
            },
            compile: compile
        };

        return directive;

        function compile(element, attrs) {

            if (attrs.required !== undefined) {
                element.find('input').attr('required', 'required');
            };

            return link;
        };

        function link(scope, element, attrs) {

            scope.name = (attrs.name !== undefined) ? attrs.name : 'audience';
            scope.locations = null;

            $alerts
                .loadMetadata()
                .then(function (result) {
                    scope.locations = result.fields.audience.choices;
                });

            scope.closeModal = closeModal;
            scope.openModal = openModal;
            scope.select = select;
            scope.toggleOpenState = toggleOpenState;

            element.on('click', openModal);

            $ionicModal.fromTemplateUrl('alert-location-choice-modal.html', {
                scope: scope,
                animation: 'slide-in-up'
            }).then(function(modal) {
                scope.modal = modal;
            });

            function select(value) {
                var
                    index = scope.ngModel.indexOf(value);

                if (index > -1) {
                    scope.ngModel.splice(index, 1);
                } else {
                    scope.ngModel.push(value);
                };
            };

            function openModal() {
                scope.modal.show();
            };

            function closeModal() {
                scope.modal.hide();
            };

            function toggleOpenState($event) {

                var
                    container = angular.element($event.currentTarget).toggleClass('closed').next().toggleClass('closed');

                if (container.hasClass('closed')) {

                    container.slideUp(function() {
                        $ionicScrollDelegate.resize();
                    });
                } else {

                    container.slideDown(function() {
                        $ionicScrollDelegate.resize();
                    });
                };
            };
        };
    };

})(angular, ionic);


