(function(angular, ionic, undefined) {

    angular
        .module('headsup.alerts')
        .service('$pushedAlertsManager', PushedAlertsManager);

    PushedAlertsManager.$inject = ['$rootScope', '$utilities'];

    function PushedAlertsManager($rootScope, $utilities) {

        var
            self = this,
            stack = [],
            notified = {};

        this.has = has;
        this.get = get;
        this.count = count;
        this.clear = clear;

        $rootScope.$on('pushNotification:received', onPushNotificationReceived);
        $rootScope.$on('alerts:latest-loaded', clear);

        function has() {
            return stack.length > 0;
        };

        function get() {
            return stack;
        };

        function count() {
            return stack.length;
        };

        function clear() {

            if (stack.length > 0) {
                stack = [];
                $rootScope.$broadcast('alerts:notification-stack:empty');
            };

            return self;
        };

        function onPushNotificationReceived($event, payload) {

            if (payload.additionalData && payload.additionalData.type == 'alert' && !notified[payload.additionalData.id]) {

                notified[payload.additionalData.id] = true;

                var
                    alert = {
                        id: parseInt(payload.additionalData.id),
                        title: payload.additionalData.title,
                        details: payload.additionalData.details,
                        created: payload.additionalData.created,
                        category: parseInt(payload.additionalData.category)
                    };

                $utilities.objectPropertiesStringToDate(alert, 'created');

                stack.unshift(alert);

                $rootScope.$broadcast('alerts:notification-stack:push', alert);

            };
        };
    };


})(angular, ionic);


