(function(angular, ionic, undefined) {

    angular
        .module('headsup.assets')
        .controller('ShowAssetController', ShowAssetController);

    ShowAssetController.$inject = ['$stateParams', '$scope', '$deviceManager', '$cordovaSocialSharing', '$remoteFileOpener', '$q', '$controller'];

    function ShowAssetController($stateParams, $scope, $deviceManager, $cordovaSocialSharing, $remoteFileOpener, $q, $controller) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var vm = this;

        vm.share = share;
        vm.open = open;

        $scope.$on('$ionicView.enter', onIonicViewEnter);

        function share(title, url) {

            if ($deviceManager.isBrowser()) {
                $deviceManager.alert('Sharing:\n\nTitle: Heads Up\nTitle: ' + title + '\nUrl: ' + url);
            } else {
                $cordovaSocialSharing.share(title, 'Heads Up', null, url);
            };
        };

        function open($event, url, mime) {

            if ($deviceManager.isBrowser()) {
                window.open(url);
                return;
            };

            vm.loadingSpinner = true;

            var
                button = angular.element($event.currentTarget);

            $remoteFileOpener
                .has(url)
                .then(function(has) {

                    var
                        deferred = $q.defer();

                    if (has) {
                        deferred.resolve();
                        vm.loadingSpinner = false;
                    } else {
                        $remoteFileOpener
                            .download(url)
                            .then(function() {
                                deferred.resolve();
                            })
                            .catch(function() {
                                deferred.reject();
                            })
                            .finally(function() {
                                vm.loadingSpinner = false;
                            });
                    }

                    return deferred.promise;
                })
                .then(function() {
                    return $remoteFileOpener.open(url, mime);
                })
                .catch(function() {
                    $deviceManager.alert('Sorry, it seems that requested file could not be opened on this device....');

                    button
                        .removeClass('button-balanced')
                        .removeClass('ion-ios-cloud-download')
                        .addClass('button-assertive')
                        .addClass('disabled')
                        .addClass('ion-close')
                        .prop('disabled', 'true')
                    ;
                });
        };

        function onIonicViewEnter() {

            vm.url = $stateParams.url;
            vm.mime = $stateParams.mime;
            vm.title = $stateParams.title;

            switch ($stateParams.mime) {
                case 'image/jpg':       // FALL TROUGH !!!
                case 'image/jpeg':      // FALL TROUGH !!!
                case 'image/gif':       // FALL TROUGH !!!
                case 'image/png':
                    vm.img = $stateParams.url;
                    break;
                default:
                    /* noop */
                    break;
            };
        };
    };

})(angular, ionic);
