(function(angular, ionic, undefined) {

    angular
        .module('geomarking')
        .directive('geomarkingEditor', geomarkingEditor);

    geomarkingEditor.$inject = ['$geomarking'];

    function geomarkingEditor($geomarking) {

        var directive = {
            restrict: 'E',
            scope: {
                ngModel: "=",
                defaultLat: "=?",
                defaultLng: "=?",
                defaultZoom: "=?"
            },
            templateUrl: 'modules/geomarking/view/geomarking-editor.directive.html',
            compile: compile,
            controller: GeomarkingDrawingCanvasController,
            controllerAs: 'vm'
        };

        return directive;

        function compile(element, attributes, transclude) {

            element.find('geomarking-editor-canvas').css({
                'height': attributes.height ? attributes.height : (300 + 'px')
            });
        };
    };

    GeomarkingDrawingCanvasController.$inject = ['$scope', '$element', '$timeout'];

    function GeomarkingDrawingCanvasController($scope, $element, $timeout) {

        var
            vm = this,
            container = $element.find('geomarking-autosuggest'),
            input = container.find('input'),
            button = container.find('button');

        vm.drawingMode = GeomarkableMap.drawingMode;
        vm.map = null;
        vm.ngModel = $scope.ngModel;
        vm.defaultLat = $scope.defaultLat;
        vm.defaultLng = $scope.defaultLng;
        vm.defaultZoom = $scope.defaultZoom;
        vm.autosuggestedPlace = null;

        vm.setDrawingMode = setDrawingMode;
        vm.deleteSelected = deleteSelected;
        vm.setMap = setMap;

        $scope.$watch('vm.autosuggestedPlace', onAutosuggestedPlaceChanged);
        button.on('click', onAddMarkerFromAutosuggest);

        function setDrawingMode(mode) {
            vm.map.enterDrawingMode(mode);
        };

        function deleteSelected() {

            if (vm.map.hasSelection()) {
                vm.map.deleteSelected();
            };
        };

        function setMap(map) {
            vm.map = map;

            vm.map
                .addEventListener('geomarkerSelected', onGeomarkerSelected)
                .addEventListener('geomarkerDeselected', onGeomarkerDeselected)
                .addEventListener('geomarkersCleared', onDrawingModeChanged)
                .addEventListener('drawingModeChanged', onDrawingModeChanged);
        };

        function onGeomarkerSelected(geomarker) {
            $element.find('toolbar-item-delete').addClass('active');
        };

        function onGeomarkerDeselected(geomarker) {
            $element.find('toolbar-item-delete').removeClass('active');
        };

        function onDrawingModeChanged(mode) {
            $element.find('geomarking-toolbar').children().removeClass('active');
            $element.find('toolbar-item-' + ((null !== mode && undefined !== mode) ? mode.toUpperCase() : 'MOVE')).addClass('active');
        };

        function onAutosuggestedPlaceChanged() {

            button.prop('disabled', true);

            if (vm.autosuggestedPlace && vm.autosuggestedPlace.geometry) {

                button.prop('disabled', false);

                if (vm.autosuggestedPlace.geometry.viewport) {
                    vm.map.setViewport(vm.autosuggestedPlace.geometry.viewport);
                } else {
                    vm.map
                        .setCenter(vm.autosuggestedPlace.geometry.location)
                        .setZoom(vm.map.getConfiguration('zoom'))
                };
            };
        };

        function onAddMarkerFromAutosuggest() {

            if (vm.autosuggestedPlace && vm.autosuggestedPlace.geometry) {

                vm.map.addMarker({
                    lat: vm.autosuggestedPlace.geometry.location.lat(),
                    lng: vm.autosuggestedPlace.geometry.location.lng(),
                    type: google.maps.drawing.OverlayType.MARKER
                });

                vm.map.setCenter(vm.autosuggestedPlace.geometry.location);
            };

            vm.autosuggestedPlace = null;
            input.val('');
            button.prop('disabled', true);
        };
    };

})(angular, ionic);
