(function(angular, ionic, undefined) {

    angular
        .module('geomarking')
        .directive('geomarkingMap', geomarkingMap);

    geomarkingMap.$inject = ['$geomarking'];

    function geomarkingMap($geomarking) {

        var directive = {
            restrict: 'E',
            scope: {
                ngModel: "=",
                defaultLat: "=?",
                defaultLng: "=?",
                defaultZoom: "=?"
            },
            require: '?^^geomarkingEditor',
            compile: compile
        };

        return directive;

        function compile(element, attrs) {

            element
                .empty();

            return function (scope, element, attrs, geomarkingEditorController) {

                var
                    map = new GeomarkableMap(element, {
                        zoom: $geomarking.getDefaultGoogleMapZoom(),
                        lat: $geomarking.getDefaultMapCenterLatitude(),
                        lng: $geomarking.getDefaultMapCenterLongitude(),
                        mapTypeControl: false,
                        editable: attrs.editable === 'true'
                    });

                if (geomarkingEditorController) {
                    geomarkingEditorController.setMap(map);
                };

                link(scope, map, element);
            };
        };

        function link(scope, map, element) {

            if (undefined !== scope.defaultZoom) {
                map.setZoom(scope.defaultZoom);
            };

            if (undefined !== scope.defaultLat && undefined !== scope.defaultLng) {
                map.setCenter(scope.defaultLat, scope.defaultLng);
            };

            map
                .setGeomarkers(scope.ngModel)
                .fitCurrentShapes()
                .addEventListener('geomarkerAdded', onGeomarkerAdded)
                .addEventListener('geomarkerRemoved', onGeomarkerRemoved)
                .addEventListener('geomarkersCleared', onGeomarkersCleared)
                .addEventListener('geomarkerSelected', onGeomarkerSelected)
                .addEventListener('geomarkerClicked', onGeomarkerClicked)
            ;

            function onGeomarkerAdded(geomarker) {
                scope.ngModel.push(geomarker);
            };

            function onGeomarkerRemoved(geomarker) {
                var
                    index = scope.ngModel.indexOf(geomarker);

                if (index > -1) {
                    scope.ngModel.splice(index, 1);
                };
            };

            function onGeomarkersCleared() {
                scope.ngModel = [];
            };

            function onGeomarkerSelected(geomarker) {
                scope.$emit('geomarkerSelected', geomarker);
            };

            function onGeomarkerClicked(geomarker) {
                scope.$emit('geomarkerClicked', geomarker);
            };
        };
    };

})(angular, ionic);

