(function(angular, ionic, undefined) {

    angular
        .module('geomarking')
        .provider('$geomarking', GeomarkingProvider);

    var
        _googleMapHeadJsScriptAttached = false,     // Did we attached google maps js inclusion in HEAD?
        _googleMapLibraryLoaded = false,            // Did google map libarary has been loaded?
        _googleMapLoadingStartTime = null;          // When did we started with google maps loading?

    function GeomarkingProvider() {

        var
            self = this;

        this.disableGoogleMapLibraryAutoload = false;
        this.googleMapLibraryUrl = 'https://maps.googleapis.com/maps/api/js';
        this.googleMapLibraryLibraries = ['drawing', 'places'];
        this.googleMapLibraryApi = null;
        this.defaultGoogleMapZoom = 17;
        this.defaultMapCenterLatitude = 0;
        this.defaultMapCenterLongitude = 0;

        this.$get = ['$window', '$q', $get];

        function $get($window, $q) {

            if (self.disableGoogleMapLibraryAutoload) {
                _googleMapHeadJsScriptAttached = true;
            };

            return new Geomarking($window, $q, {
                googleMapLibraryUrl: (function(url, libraries, apiKey) {

                    url = url + '?';

                    if (apiKey) {
                        url = url + 'key=' + apiKey + '&';
                    };

                    return url + 'libraries=' + libraries.join(',');

                })(self.googleMapLibraryUrl, self.googleMapLibraryLibraries, self.googleMapLibraryApi),
                defaultGoogleMapZoom: self.defaultGoogleMapZoom,
                defaultMapCenterLatitude: self.defaultMapCenterLatitude,
                defaultMapCenterLongitude: self.defaultMapCenterLongitude
            });
        };
    };

    function Geomarking($window, $q, settings) {

        this.getDefaultGoogleMapZoom = getDefaultGoogleMapZoom;
        this.getDefaultMapCenterLatitude = getDefaultMapCenterLatitude;
        this.getDefaultMapCenterLongitude = getDefaultMapCenterLongitude;
        this.loadGoogleMapLibrary = loadGoogleMapLibrary;

        /**
         * Get default map center longitude.
         *
         * @return {number}
         */
        function getDefaultMapCenterLongitude() {
            return settings.defaultMapCenterLongitude;
        };

        /**
         * Get default map center latitude.
         *
         * @return {number}
         */
        function getDefaultMapCenterLatitude() {
            return settings.defaultMapCenterLatitude;
        };

        /**
         * Get default Google map zoom.
         *
         * @return {int}
         */
        function getDefaultGoogleMapZoom() {
            return settings.defaultGoogleMapZoom;
        };

        /**
         * Load google maps library asynchronously
         *
         * @return {$q.promise}
         */
        function loadGoogleMapLibrary() {

            var
                deferred = $q.defer();

            if (!_googleMapHeadJsScriptAttached) {

                var
                    el = $window.document.createElement('script');
                el.src = settings.googleMapLibraryUrl;

                angular.element(document.getElementsByTagName('head')).append(el);

                _googleMapHeadJsScriptAttached = true;
            };

            if (!_googleMapLibraryLoaded) {
                if (!_googleMapLoadingStartTime) {
                    _googleMapLoadingStartTime = new Date().getTime() / 1000;
                };

                function waitOnLoad() {

                    if (typeof google === 'object' && typeof google.maps === 'object') {
                        _googleMapLibraryLoaded = true;
                        deferred.resolve();
                        return;
                    };

                    if ((_googleMapLoadingStartTime + 30) < (new Date().getTime() / 1000)) {
                        throw 'It is impossible to load Google maps library for 30 seconds.';
                    };

                    setTimeout(waitOnLoad, 300);
                };

                waitOnLoad();

            } else {
                deferred.resolve();
            };

            return deferred.promise;
        };
    };

})(angular, ionic);
