(function(angular, ionic, undefined) {

    angular
        .module('headsup.map')
        .controller('ShowAlertsMapController', ShowAlertsMapController);

    ShowAlertsMapController.$inject = ['$alerts', '$deviceManager', '$ionicHistory', '$scope', '$state', '$ionicPopover', '$controller'];

    function ShowAlertsMapController($alerts, $deviceManager, $ionicHistory, $scope, $state, $ionicPopover, $controller) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var
            vm = this,
            popover = null;

        vm.reload = reload;

        $scope.$on('$ionicView.enter', onIonicViewEnter);
        $scope.$on('geomarkerClicked', onGeomarkerClicked);

        function reload() {

            vm.nbSubscribedCategories = $deviceManager.getSubscribedCategories().length;
            vm.nbSubscribedLocations = $deviceManager.getSubscribedLocations().length;

            if (vm.nbSubscribedCategories > 0 && vm.nbSubscribedLocations > 0) {

                $alerts.map().then(function(geomarkers) {
                    vm.loadingSpinner = false;
                    vm.geomarkers = geomarkers;
                });
            } else {
                vm.loadingSpinner = false;
                vm.geomarkers = [];
            };

            $scope.$broadcast('scroll.refreshComplete');

        };

        function onIonicViewEnter() {


            if (!$ionicHistory.forwardView()
                ||
                (
                    $ionicHistory.forwardView().stateName !== 'app.alert'
                    &&
                    $ionicHistory.forwardView().stateName !== 'app.login'
                )
            ) {
                vm.reload();
                vm.loadingSpinner = true;
            };
        };

        function onGeomarkerClicked(event, geomarker) {

            if (popover) {
                popover.remove();
                popover = null;
            };

            var
                popoverScope = $scope.$new();

            popoverScope.alert = geomarker.alert;
            popoverScope.show = show;

            $ionicPopover.fromTemplateUrl('modules/map/view/map-popover.html', {
                scope: popoverScope,
                animation: 'slide-in-up'
            }).then(function(result){
                popover = result;
                popover.show();
            });
        };

        function show(alert) {

            if (popover) {
                popover.remove();
                popover = null;
            };

            $state.go('app.alert', {
                alertId: alert.id
            });
        };

    };

})(angular, ionic);
