(function(angular, ionic, undefined) {

    angular
        .module('headsup.settings')
        .directive('settingsTree', settingsTree)
        .directive('settingsNode', settingsNode);

    function settingsTree() {

        var directive = {
            template: '<ul class="settings-tree"><settings-node ng-repeat="node in tree"></settings-node></ul>',
            replace: true,
            transclude: true,
            restrict: 'E',
            scope: {
                tree: '=ngModel',
                parentNode: '=?ngParent'
            }
        };

        return directive;
    };

    settingsNode.$inject = ['$compile', '$ionicScrollDelegate', '$deviceManager'];

    function settingsNode($compile, $ionicScrollDelegate, $deviceManager) {

        var directive  = {
            restrict: 'E',
            replace:true,
            templateUrl: 'modules/settings/view/directive/settings-tree.directive.html',
            link: link
        };

        return directive;

        function link(scope, elm, attrs) {

            scope.toggleNode = toggleNode;
            scope.toggleValue = toggleValue;
            scope.node.subscribed = isSubscribed(scope.node);

            if (scope.node.children.length > 0) {

                var
                    childNode = $compile('<settings-tree ng-model="node.children"></settings-tree>')(scope)

                elm.append(childNode);
                childNode.addClass('closed');
            };

            function toggleNode($event) {
                var
                    element = angular.element($event.currentTarget),
                    container = element.parent().parent().next();

                element.children().toggleClass('open');

                container.toggleClass('closed');

                if (container.hasClass('closed')) {

                    container.slideUp(function() {
                        $ionicScrollDelegate.resize();
                    });
                } else {

                    container.slideDown(function() {
                        $ionicScrollDelegate.resize();
                    });
                };
            };

            function toggleValue(node, value) {
                switch (node.type) {
                    case 'audience':

                        if (value) {
                            $deviceManager.subscribeToLocation(node.id);
                        } else {
                            $deviceManager.unsubscribeFromLocation(node.id);
                        };

                        break;
                    case 'category':

                        if (value) {
                            $deviceManager.subscribeToCategory(node.id);
                        } else {
                            $deviceManager.unsubscribeFromCategory(node.id);
                        };

                        break;
                    default:
                        throw 'Unknown node type: ' + node.type + '.';
                        break;
                };
            };

            function isSubscribed(node) {

                switch (node.type) {
                    case 'audience':
                        return $deviceManager.isSubscribedToLocation(node.id);
                        break;
                    case 'category':
                        return $deviceManager.isSubscribedToCategory(node.id);
                        break;
                    default:
                        throw 'Unknown node type: ' + node.type + '.';
                        break;
                };
            };
        };
    };




})(angular, ionic);
