(function(angular, ionic, undefined) {

    angular
        .module('headsup.settings')
        .controller('SettingsController', SettingsController);

    SettingsController.$inject = ['$categories', '$locations', '$deviceManager', '$ionicScrollDelegate', '$controller', '$scope'];


    function SettingsController($categories, $locations, $deviceManager, $ionicScrollDelegate, $controller, $scope) {

        angular.extend(this, $controller('PushedAlertAbstractController', { $scope: $scope, options: { withPopup: true }}));

        var vm = this;

        vm.categories = $categories.getAll();
        vm.locations = $locations.getAll();
        vm.categoriesCount = $categories.count();
        vm.locationsCount = $locations.count();

        vm.toggleLocation = toggleLocation;
        vm.toggleCategory = toggleCategory;
        vm.toggleSubitems = toggleSubitems;
        vm.isSubscribedToLocation = isSubscribedToLocation;
        vm.isSubscribedToCategory = isSubscribedToCategory;

        function toggleLocation(id, value) {
            if (value) {
                $deviceManager.subscribeToLocation(id);
            } else {
                $deviceManager.unsubscribeFromLocation(id);
            };
        };

        function toggleCategory(id, value) {
            if (value) {
                $deviceManager.subscribeToCategory(id);
            } else {
                $deviceManager.unsubscribeFromCategory(id);
            };
        };

        function toggleSubitems(event) {
            angular.element(event.target).parent().toggleClass('closed');
            $ionicScrollDelegate.resize();
        };

        function isSubscribedToLocation(id) {
            return $deviceManager.isSubscribedToLocation(id)
        };

        function isSubscribedToCategory(id) {
            return $deviceManager.isSubscribedToCategory(id);
        };

    };

})(angular, ionic);