(function(angular, ionic, undefined) {

    angular
        .module('headsup')
        .factory('$jwtUserHttpInterceptor', jwtUserHttpInterceptor);

    jwtUserHttpInterceptor.$inject = ['$q', '$rootScope'];

    function jwtUserHttpInterceptor($q, $rootScope) {

        var jwtToken = null;

        var interceptingFunctions = {
            request: onRequest,
            response: onResponse,
            responseError: onResponseError
        };

        return interceptingFunctions;

        function onRequest(config) {

            if (null !== jwtToken) {
                config.headers['Authorization'] = 'Bearer ' + jwtToken;
            };

            return config;
        };

        function onResponseError(rejection) {

            if (401 == rejection.status) {

                jwtToken = null;

                // Shut it down temporarily
                interceptingFunctions.responseError = null;

                var
                    deferred = $q.defer();

                $rootScope.$broadcast('headsup:jwt:handle_token_expired', deferred, rejection);

                deferred
                    .promise
                    .finally(function () {
                        interceptingFunctions.responseError = onResponseError;
                    });

                return deferred.promise;
            };

            return $q.reject(rejection);
        };


        function onResponse(response) {

            var
                headers = response.headers();

            if (headers['x-jwt-token']) {

                jwtToken = (headers['x-jwt-token'] && 'void' !== headers['x-jwt-token']) ? headers['x-jwt-token'] : null;

                if (null === jwtToken) {
                    // @TODO auto login!
                };
            };

            return response;
        };
    }

})(angular, ionic);
