(function(angular, ionic, undefined) {

    angular
        .module('headsup.user')
        .service('$userAccount', UserAccount);

    UserAccount.$inject = ['$http', '$q', '$utilities', '$deviceManager', '$localStorage', '$sessionStorage', '$rootScope'];

    function UserAccount($http, $q, $utilities, $deviceManager, $localStorage, $sessionStorage, $rootScope) {

        $rootScope.$on('headsup:jwt:handle_token_expired', handleTokenExpired);

        var
            self = this;

        this.login = function(username, password) {

            var
                deferred = $q.defer();

            $sessionStorage.clear();

            $http({
                method: 'POST',
                url: $utilities.getApiUrl('/user/login/v1'),
                headers: {
                    'Content-Type': 'application/json'
                },
                data: {
                    "deviceType": $deviceManager.getPlatform(),
                    "deviceId": $deviceManager.getUuid(),
                    "deviceApp": $deviceManager.getApp(),
                    "username": username,
                    "password": password
                }
            }).then(function(result) {
                if (result.data.error) {

                    return deferred.reject({
                        message: 'Unable to login user.',
                        reason: {
                            message: result.data.message
                        }
                    });
                } else {

                    $localStorage.save('user.username', username);
                    $localStorage.save('user.password', password);
                    $localStorage.save('user.authenticated', true);

                    $rootScope.$broadcast('headsup:user:login');

                    return deferred.resolve(true);
                };
            }, function(reason) {

                return deferred.reject({
                    message: 'Unable to login user.',
                    reason: reason
                });
            });

            return deferred.promise;
        };

        this.logout = function() {

            var
                deferred = $q.defer();

            $http({
                method: 'GET',
                url: $utilities.getApiUrl('/user/logout/v1')
            }).then(function(result) {

                $localStorage.save('user.username', null);
                $localStorage.save('user.password', null);
                $localStorage.save('user.authenticated', false);
                $sessionStorage.clear();

                deferred.resolve(true);

                $rootScope.$broadcast('headsup:user:logout');
                
            }, function(reason) {

                return deferred.reject({
                    message: 'Unable to logout user.',
                    reason: reason
                });
            });

            return deferred.promise;
        };

        this.status = function() {

            var
                deferred = $q.defer();

            $http({
                method: 'GET',
                url: $utilities.getApiUrl('/user/status/v1')
            }).then(function(result) {
                return deferred.resolve(result.data.loggedIn);
            }, function(reason) {
                return deferred.reject({
                    message: 'Unable to get login status',
                    reason: reason
                });
            });

            return deferred.promise;
        };

        this.isAuthenticated = function() {
            return $localStorage.get('user.authenticated', false);
        };

        this.getSavedUsername = function() {
            return $localStorage.get('user.username', null);
        };

        this.getSavedPassword = function() {
            return $localStorage.get('user.password', null);
        };

        function handleTokenExpired(event, deferred, rejection) {

            if (!self.isAuthenticated()) {
                return deferred.reject(rejection);
            };

            self
                .login(self.getSavedUsername(), self.getSavedPassword())
                .then(function () {

                    $http(rejection.config)
                        .then(function (response) {
                            deferred.resolve(response);
                        })
                        .catch(function (response) {
                            deferred.reject(response);
                        });
                })
                .catch(function () {
                    return deferred.reject(rejection)
                });
        };
    };

})(angular, ionic);
